// Copyright (c) 2003 Charles Samuels <charles@kde.org>
// See the file COPYING for redistribution terms.

#include "cmodule.h"
#include "oblique.h"

#include <kregexpeditorinterface.h>
#include <kparts/componentfactory.h>
#include <klocale.h>
#include <klistview.h>
#include <klineedit.h>
#include <kiconloader.h>
#include <klineeditdlg.h>

#include <qgroupbox.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qfileinfo.h>
#include <qtooltip.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qcombobox.h>

Configure::Configure(Oblique *parent)
	: CModule(i18n("Oblique"), i18n("Configure Oblique Playlist"), "", parent)
{
	mOblique = parent;
	mIgnore = true;

	{
		QVBoxLayout *layout = new QVBoxLayout(this);
		layout->setAutoAdd(true);
		layout->setSpacing(7);
	}


	{
		QHBox *box = new QHBox(this);
		mSchemaList = new QComboBox(box);
		connect(
				mSchemaList, SIGNAL(activated(const QString&)),
				SLOT(selectSchema(const QString&))
			);

		mAdd = new QPushButton(BarIconSet("filenew"), 0, box);
		mAdd->setFixedWidth(mAdd->height());
		QToolTip::add(mAdd, i18n("Create new schema"));
		connect(mAdd, SIGNAL(clicked()), SLOT(newSchema()));

		mRemove = new QPushButton(BarIconSet("editdelete"), 0, box);
		mRemove->setFixedWidth(mRemove->height());
		QToolTip::add(mRemove, i18n("Remove this schema"));
		connect(mRemove, SIGNAL(clicked()), SLOT(removeSchema()));

		mCopy = new QPushButton(BarIconSet("editcopy"), 0, box);
		mCopy->setFixedWidth(mCopy->height());
		QToolTip::add(mCopy, i18n("Copy this schema"));
		connect(mCopy, SIGNAL(clicked()), SLOT(copySchema()));
	}


	{
		QHBox *middle = new QHBox(this);

		mSchemaTree = new KListView(middle);
		mSchemaTree->setSorting(-1);
		connect(
				mSchemaTree, SIGNAL(currentChanged(QListViewItem*)),
				SLOT(setCurrent(QListViewItem*))
			);
		mSchemaTree->addColumn(i18n("Property"));
		mSchemaTree->addColumn(i18n("Value"));
		mSchemaTree->addColumn(i18n("Presentation"));

		QVBox *buttons = new QVBox(middle);

		mAddSibling = new QPushButton(BarIconSet("1rightarrow", KIcon::SizeSmall), "",buttons);
		mAddSibling->setFixedWidth(mAddSibling->height());
		connect(mAddSibling, SIGNAL(clicked()), SLOT(addSibling()));
		QToolTip::add(mAddSibling, i18n("Create a new item after the selected one"));

		mAddChild = new QPushButton(BarIconSet("2rightarrow", KIcon::SizeSmall), "", buttons);
		mAddChild->setFixedWidth(mAddChild->height());
		connect(mAddSibling, SIGNAL(clicked()), SLOT(addChild()));
		QToolTip::add(mAddSibling, i18n("Create a new child item under the selected one"));

		mRemoveSelf = new QPushButton(BarIconSet("filenew", KIcon::SizeSmall), "", buttons);
		mRemoveSelf->setFixedWidth(mRemoveSelf->height());
		connect(mRemoveSelf, SIGNAL(clicked()), SLOT(removeSelf()));
		QToolTip::add(mAddSibling, i18n("Remove the selected item"));
	}


	{
		QVBox *side = new QVBox(this);
		// the controllers
		{
			QWidget *topSide = new QWidget(side);
			QGridLayout *grid = new QGridLayout(topSide);

			QLabel *label;

			label = new QLabel(i18n("&Property:"), topSide);
			mPropertyEdit = new KLineEdit(topSide);
			label->setBuddy(mPropertyEdit);
			grid->addWidget(label, 0, 0);
			grid->addMultiCellWidget(mPropertyEdit, 0, 0, 1, 2);
			connect(mPropertyEdit, SIGNAL(textChanged(const QString&)), SLOT(updateCurrent()));

			label = new QLabel(i18n("&Value:"), topSide);
			mValueEdit = new KLineEdit(topSide);
			label->setBuddy(mPropertyEdit);
			grid->addWidget(label, 1, 0);
			grid->addMultiCellWidget(mValueEdit, 1, 1, 1, 1);
			connect(mValueEdit, SIGNAL(textChanged(const QString&)), SLOT(updateCurrent()));

			QPushButton *editRe = new QPushButton(i18n("&Edit..."), topSide);
			grid->addWidget(editRe, 1, 2);
			connect(editRe, SIGNAL(clicked()), SLOT(editValueRegexp()));

			label = new QLabel(i18n("Pre&sentation:"), topSide);
			mPresentationEdit = new KLineEdit(topSide);
			label->setBuddy(mPropertyEdit);
			grid->addWidget(label, 2, 0);
			grid->addMultiCellWidget(mPresentationEdit, 2, 2, 1, 2);
			connect(mPresentationEdit, SIGNAL(textChanged(const QString&)), SLOT(updateCurrent()));
		}

		{
			QGroupBox *groupbox = new QGroupBox(
					4, Qt::Horizontal, i18n("Options"), side
				);

			mOptionStopOnHit = new QCheckBox(i18n("&Stop on hit"), groupbox);
			connect(mOptionStopOnHit, SIGNAL(toggled(bool)), SLOT(updateCurrent()));
			mOptionUnique = new QCheckBox(i18n("&Unique"), groupbox);
			connect(mOptionUnique, SIGNAL(toggled(bool)), SLOT(updateCurrent()));
		}

//		side->setStretchFactor(new QWidget(side), 1);

	}

}

class QueryGroupItem : public KListViewItem
{
	QueryGroup mModified;

public:
	QueryGroupItem(QueryGroupItem *parent, QueryGroup *group)
		: KListViewItem(parent)
	{
		init(group);
	}

	QueryGroupItem(KListView *parent, QueryGroup *group)
		: KListViewItem(parent)
	{
		init(group);
	}

	QueryGroup *modified() { return &mModified; }
	const QueryGroup *modified() const { return &mModified; }

	void redisplay()
	{
		setText(0, modified()->propertyName());
		setText(1, modified()->value().pattern());
		setText(2, modified()->presentation());
	}

	QueryGroupItem *parent()
		{ return static_cast<QueryGroupItem*>(KListViewItem::parent()); }

	KListView *listView()
		{ return static_cast<KListView*>(KListViewItem::listView()); }

private:
	void init(QueryGroup *group)
	{
		mModified = *group;
		redisplay();

		if (group->firstChild())
			new QueryGroupItem(this, group->firstChild());

		// do child and siblings now
		if (QueryGroup *after = group->nextSibling())
		{
			if (parent())
				new QueryGroupItem(parent(), after);
			else
				new QueryGroupItem(listView(), after);
		}

		setOpen(true);
	}
};


void Configure::reopen()
{
	mQueries.clear();
	mSchemaList->clear();

	QStringList names = oblique()->schemaCollection().names();

	for (QStringList::Iterator i(names.begin()); i != names.end(); ++i)
	{
		QueryItem qi;
		qi.title = qi.query.load(oblique()->schemaCollection().file(*i));
		qi.changed = false;
		mQueries.insert(*i, qi);

		mSchemaList->insertItem(qi.title);
	}


}

void Configure::save()
{
	for (QMap<QString,QueryItem>::Iterator i(mQueries.begin()); i != mQueries.end(); ++i)
	{
		if (!i.data().changed) continue;

		QString name = i.key();
		name = QFileInfo(name).fileName();
		QString fn = oblique()->schemaCollection().saveFile(name);

		i.data().query.save(i.data().title, fn);

		if (oblique()->fileOfQuery() == name)
		{
			oblique()->loadQuery(name);
		}
	}
}

QString Configure::nameToFilename(const QString &_name)
{
	QString name = _name;
	return name;
}

void Configure::newSchema()
{
	bool ok;
	QString str=KLineEditDlg::getText(i18n("Name of schema:"), "", &ok, this);
	if (!ok) return;

	QString filename = nameToFilename(str);

	if (mQueries.contains(nameToFilename(filename))) return;
	QString save = oblique()->schemaCollection().saveFile(filename);

	QueryItem queryitem;
	queryitem.query = Query();
	queryitem.title = str;
	queryitem.changed=true;
	mSchemaList->insertItem(str);
	mQueries.insert(filename, queryitem);

	selectSchema(str);
}

void Configure::copySchema()
{

}

void Configure::removeSchema()
{

}

void Configure::selectSchema(const QString &title)
{
	mSchemaTree->clear();

	for (QMap<QString,QueryItem>::Iterator i(mQueries.begin()); i != mQueries.end(); ++i)
	{
		if (i.data().title != title) continue;
		if (QueryGroup *grp = i.data().query.firstChild())
		{
			new QueryGroupItem(mSchemaTree, grp);
		}

		break;
	}
}

void Configure::editValueRegexp()
{
	unless (mRegexpEditor)
	{
		mRegexpEditor =
			KParts::ComponentFactory::createInstanceFromQuery<QDialog>(
					"KRegExpEditor/KRegExpEditor", QString::null, this
				);
	}

	if ( mRegexpEditor )
	{
		KRegExpEditorInterface *iface =
			static_cast<KRegExpEditorInterface*>(
					mRegexpEditor->qt_cast( "KRegExpEditorInterface")
				);

		iface->setRegExp(mValueEdit->text());
		if (mRegexpEditor->exec() == QDialog::Accepted)
			mValueEdit->setText(iface->regExp());
	}
}

void Configure::setCurrent(QListViewItem *_item)
{
	if (!_item) return;
	QueryGroupItem *item = static_cast<QueryGroupItem*>(_item);
	mIgnore = true;
	mPropertyEdit->setText(item->modified()->propertyName());
	mValueEdit->setText(item->modified()->value().pattern());
	mPresentationEdit->setText(item->modified()->presentation());

	mOptionStopOnHit->setChecked(item->modified()->option(QueryGroup::StopOnHit));
	mOptionUnique->setChecked(item->modified()->option(QueryGroup::Unique));
	mIgnore = false;
}


void Configure::updateCurrent()
{
	QueryGroupItem *item = static_cast<QueryGroupItem*>(mSchemaTree->currentItem());
	if (mIgnore || !item) return;

	QueryGroup *mod = item->modified();

	mod->setPropertyName(mPropertyEdit->text());
	mod->setPresentation(mPresentationEdit->text());
	mod->setValue(QRegExp(mValueEdit->text()));

	mod->setOption(QueryGroup::StopOnHit, mOptionStopOnHit->isChecked());
	mod->setOption(QueryGroup::Unique, mOptionUnique->isChecked());
	item->redisplay();
}

void Configure::addSibling()
{

}


void Configure::addChild()
{

}

void Configure::removeSelf()
{

}


#include "cmodule.moc"

